using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// ShardsEditor v1.0
//
// Allows you to edit some parameters of dust particles in the package Shards 101

namespace ShardsEditorPlugin
{
    public class ShardsEditor : MVRScript
    {
		public JSONStorableColor ShardsMainColor;
		public JSONStorableFloat ShardsMainOpacity;
		public JSONStorableFloat ShardsMainMaxParticles;
		public JSONStorableFloat ShardsMainSimulationSpeed;
		public JSONStorableFloat ShardsVelocitySpeedModifier;
		
		public Transform ParticleRoot;
		private ParticleSystem prtComponent;
		private string particleSuffixName = "_editable";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add ShardsEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }
				
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				ShardsMainColor = new JSONStorableColor("Particles color", hsvc, MainColorCallback);
				RegisterColor(ShardsMainColor);
				CreateColorPicker(ShardsMainColor);
				
				ShardsMainOpacity = new JSONStorableFloat("Particles opacity", 1.0f, MainOpacityCallback, 0f, 1f);
				RegisterFloat(ShardsMainOpacity);
				CreateSlider(ShardsMainOpacity, false);
				
				ShardsMainMaxParticles = new JSONStorableFloat("Max particles", 5000, MainMaxParticlesCallback, 1, 5000);
				RegisterFloat(ShardsMainMaxParticles);
				CreateSlider(ShardsMainMaxParticles, false);
				
				ShardsMainSimulationSpeed = new JSONStorableFloat("Simulation speed", 1, MainSimulationSpeedCallback, 1, 20);
				RegisterFloat(ShardsMainSimulationSpeed);
				CreateSlider(ShardsMainSimulationSpeed, false);
				
				ShardsVelocitySpeedModifier = new JSONStorableFloat("Particles speed modifier", 1, VelocitySpeedModifierCallback, 0.5f, 20, true);
				RegisterFloat(ShardsVelocitySpeedModifier);
				CreateSlider(ShardsVelocitySpeedModifier, false);
				
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>ShardsEditor</b> is a tiny plugin that allows you to edit some parameters of the shards particles inside Shards 101 v1.\n\n" +
					"Just test the parameters on the left, and the particle will update in realtime.\n\n" +
					"Don't be eager to see the changes! Particles have a lifetime of around 5 to 20 seconds each, the change will progressively appear after their lifecycle ends.\n\n" +
					"Same when you load a scene, the settings stored in your scene will take a few moments to apply as it is also tied to particles lifecycle.\n\n" +
					"<b>For creators and screenshot/still artists :</b> Be careful. I cannot explain why, but the vanilla screenshot feature of VAM does not render exactly the same as what you see on the realtime viewport. " +
					"In my case, I used virtual resolution to upscale the game and make higher res screenshots.\n\n" +
					"<b>IMPORTANT :</b> If you switch particles in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("VAMLock - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitDE();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("ShardsEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitDE() {
			if( seInitialized == true ) return true;
			
			ParticleRoot = getParticleRoot( containingAtom.gameObject.transform );
			if( ParticleRoot == null ) {
				return false;
			}
			
			prtComponent = ParticleRoot.GetComponent<ParticleSystem>();
			if( prtComponent ) {		
				seInitialized = true;
				
				MainColorCallback( ShardsMainColor );
				MainOpacityCallback( ShardsMainOpacity.val );
				MainMaxParticlesCallback( ShardsMainMaxParticles.val );
				MainSimulationSpeedCallback( ShardsMainSimulationSpeed.val );
				VelocitySpeedModifierCallback( ShardsVelocitySpeedModifier.val );
				
				logDebug("ShardsEditor : Particle editor initialized properly");
							
				return true;
			} else {
				logError("ShardsEditor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getParticleRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(particleSuffixName) ) {
					return child;
				} else {
					return getParticleRoot( child );
				}
			}
			
			return null;
		}
		
		protected void MainColorCallback(JSONStorableColor color) {
			if( prtComponent ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, ShardsMainOpacity.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = newColor;
			}
		}
		
		protected void MainOpacityCallback(float opacity) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startColor = new Color(ShardsMainColor.colorPicker.currentColor.r, ShardsMainColor.colorPicker.currentColor.g, ShardsMainColor.colorPicker.currentColor.b, opacity);
			}
		}
		
		protected void MainMaxParticlesCallback(float maxprts) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.maxParticles = (int)maxprts;
			}
		}
		
		protected void MainSimulationSpeedCallback(float simspeed) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.simulationSpeed = simspeed;
			}
		}
		
		protected void VelocitySpeedModifierCallback(float speedmult) {
			if( prtComponent ) {
				var prtVelocity = prtComponent.velocityOverLifetime;
				prtVelocity.speedModifier = speedmult;
			}
		}	
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}